@@ -1,8 +1,8 @@
 /*
- *  802.11 40/104 bit WEP / WPA-PSK Key Cracker
+ *  802.11 WEP / WPA-PSK Key Cracker
  *
  *  Copyright (C) 2006 Thomas d'Otreppe
- *  Original work: Christophe Devine
+ *  Copyright (C) 2004,2005 Christophe Devine
  *
  *  Advanced WEP attacks developed by KoreK
  *  WPA-PSK  attack code developed by Joshua Wright
@@ -67,6 +67,8 @@ extern int shammx_data( uchar ctx[40], uchar data[128], uchar buf[640] )
 
 #endif
 
+extern char * getVersion(char * progname, int maj, int min, int submin, int betavers);
+
 #define BROADCAST "\xFF\xFF\xFF\xFF\xFF\xFF"
 
 static uchar ZERO[32] =
@@ -82,7 +84,8 @@ struct options
     int bssid_set;              /* bssid set flag       */
     char essid[33];             /* target ESSID         */
     uchar bssid[6];             /* target BSSID         */
-    int nbcpu;                  /* # of cracker threads */
+    int nbcpu;                  /* # of cracker threads 
+    								(= # of CPU)*/
     int is_quiet;               /* quiet mode flag      */
 
     uchar debug[64];            /* user-defined WEP key */
@@ -93,7 +96,7 @@ struct options
     int korek;                  /* attack strategy      */
     int is_alnum;               /* alphanum keyspace    */
     int is_bcdonly;             /* binary coded decimal */
-    int do_brute;               /* bruteforce last 2    */
+    int do_brute;               /* bruteforce last 2 KB */
     int do_testy;               /* experimental attack  */
 
     FILE *dict;                 /* dictionary file      */
@@ -211,17 +214,41 @@ int K_COEFF[N_ATTACKS] =
     15, 13, 12, 12, 12, 5, 5, 5, 3, 4, 3, 4, 3, 13, 4, 4, -20
 };
 
+const uchar R[256] = 
+{
+	   0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20
+	, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40
+	, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60
+	, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80
+	, 81, 82, 83, 84, 85, 86, 87, 88, 89, 90, 91, 92, 93, 94, 95, 96, 97, 98, 99, 100
+	, 101, 102, 103, 104, 105, 106, 107, 108, 109, 110, 111, 112, 113, 114, 115, 116
+	, 117, 118, 119, 120, 121, 122, 123, 124, 125, 126, 127, 128, 129, 130, 131, 132
+	, 133, 134, 135, 136, 137, 138, 139, 140, 141, 142, 143, 144, 145, 146, 147, 148
+	, 149, 150, 151, 152, 153, 154, 155, 156, 157, 158, 159, 160, 161, 162, 163, 164
+	, 165, 166, 167, 168, 169, 170, 171, 172, 173, 174, 175, 176, 177, 178, 179, 180
+	, 181, 182, 183, 184, 185, 186, 187, 188, 189, 190, 191, 192, 193, 194, 195, 196
+	, 197, 198, 199, 200, 201, 202, 203, 204, 205, 206, 207, 208, 209, 210, 211, 212
+	, 213, 214, 215, 216, 217, 218, 219, 220, 221, 222, 223, 224, 225, 226, 227, 228
+	, 229, 230, 231, 232, 233, 234, 235, 236, 237, 238, 239, 240, 241, 242, 243, 244
+	, 245, 246, 247, 248, 249, 250, 251, 252, 253, 254, 255
+};
+
 char usage[] =
 
+"\n"
+"  %s - (C) 2006 Thomas d\'Otreppe\n"
+"  Original work: Christophe Devine\n"
+"  http://www.aircrack-ng.org\n"
+"\n"
+"  usage: aircrack-ng [options] <.cap / .ivs file(s)>\n"
 "\n"
 "  Common options:\n"
 "\n"
 "      -a <amode> : force attack mode (1/WEP, 2/WPA-PSK)\n"
 "      -e <essid> : target selection: network identifier\n"
-"      -b <bssid> : target selection: access point's MAC\n"
-"      -p <nbcpu> : SMP support: # of processes to start\n"
+"      -b <bssid> : target selection: access point's MAC"
+"%s"
 "      -q         : enable quiet mode (no status output)\n"
-"      -w <words> : path to a dictionary file\n"
 "\n"
 "  Static WEP cracking options:\n"
 "\n"
@@ -229,21 +256,22 @@ char usage[] =
 "      -t         : search binary coded decimal chr only\n"
 "      -d <start> : debug - specify beginning of the key\n"
 "      -m <maddr> : MAC address to filter usable packets\n"
-"      -n <nbits> : WEP key length: 64 / 128 / 152 / 256\n"
+"      -n <nbits> : WEP key length :  64/128/152/256/512\n"
 "      -i <index> : WEP key index (1 to 4), default: any\n"
 "      -f <fudge> : bruteforce fudge factor,  default: 2\n"
 "      -k <korek> : disable one attack method  (1 to 17)\n"
-"      -x         : do bruteforce the  last two keybytes\n"
+"      -x         : disable two last keybytes bruteforce\n"
 "      -y         : experimental  single bruteforce mode\n"
 "\n"
-"  %s - (C) 2006 Thomas d\'Otreppe\n"
-"  Original work: Christophe Devine\n"
-"  http://www.aircrack-ng.org\n"
+"  WPA-PSK cracking options:\n"
 "\n"
-"  usage: aircrack-ng [options] <.cap / .ivs file(s)>\n"
+"      -w <words> : path to a dictionary file\n"
+/*"      -r <table> : path to a WPA PMK table\n" */
 "\n";
 
 
+char * progname;
+
 void eof_wait( int *eof_notified )
 {
     if( *eof_notified == 0 )
@@ -592,7 +620,7 @@ add_wep_iv:
 
                     if( ap_cur->ivbuf == NULL )
                     {
-                        perror( "remalloc failed" );
+                        perror( "realloc failed" );
                         break;
                     }
                 }
@@ -939,7 +967,7 @@ int safe_write( int fd, void *buf, size_t len )
 int crack_wep_thread( void *arg )
 {
     long xv, min, max;
-    uchar R[256], jj[256];
+    uchar jj[256];
     uchar S[256], Si[256];
     uchar K[64];
 
@@ -950,9 +978,6 @@ int crack_wep_thread( void *arg )
     int i, j, B, cid = (long) arg;
     int votes[N_ATTACKS][256];
 
-    for( i = 0; i < 256; i++ )
-        R[i] = i;
-
     while( 1 )
     {
         if( safe_read( mc_pipe[cid][0], (void *) &B,
@@ -979,7 +1004,7 @@ int crack_wep_thread( void *arg )
             memcpy( S,  R, 256 );
             memcpy( Si, R, 256 );
 
-            for( i = j = 0; i < q; i++ )
+            for( i = j = 0; i < q; ++i )
             {
                 jj[i] = j = ( j + S[i] + K[i % (3 + opt.keylen)] ) & 0xFF;
                 SWAP( S[i], S[j] );
@@ -1191,8 +1216,8 @@ void show_wep_stats( int B, int force )
     if( opt.l33t )
         printf( "\33[34;1m" );
 
-    printf( "\33[2;%dHaircrack %d.%d\n\n", (ws.ws_col - 12) / 2,
-            _MAJ, _MIN );
+    printf( "\33[2;%dH%s\n\n", (ws.ws_col - 12) / 2, 
+    	progname );
 
     if( opt.l33t )
         printf( "\33[33;1m" );
@@ -1205,7 +1230,7 @@ void show_wep_stats( int B, int force )
 
     printf( "\33[7;4HKB    depth   byte(vote)\n" );
 
-    for( i = 0; i <= B; i++ )
+    for( i = 0; i <= B; ++i )
     {
         int j, k = ( ws.ws_col - 20 ) / 9;
 
@@ -1216,7 +1241,7 @@ void show_wep_stats( int B, int force )
             printf( "   %2d  %3d/%3d   ",
                     i, wep.depth[i], wep.fudge[i] );
 
-        for( j = wep.depth[i]; j < k + wep.depth[i]; j++ )
+        for( j = wep.depth[i]; j < k + wep.depth[i]; ++j )
         {
             if( j >= 256 ) break;
 
@@ -1253,30 +1278,28 @@ void show_wep_stats( int B, int force )
 
 int check_wep_key( int B )
 {
-    uchar K[64];
-    uchar S[256];
-    uchar R[256];
     uchar x1, x2;
     unsigned long xv;
     int i, j, n, bad;
     int nb_ascii;
 
+    uchar K[64];
+    uchar S[256];
+
+
     nb_tried++;
     bad = 0;
 
     memcpy( K + 3, wep.key, opt.keylen );
 
-    for( i = 0; i < 256; i++ )
-        R[i] = i;
-
-    for( n = 0; n < 32; n++ )
+    for( n = 0; n < 32; ++n )
     {
         xv = 5 * ( rand() % wep.nb_ivs );
 
         memcpy( K, &wep.ivbuf[xv], 3 );
         memcpy( S, R, 256 );
 
-        for( i = j = 0; i < 256; i++ )
+        for( i = j = 0; i < 256; ++i )
         {
             j = ( j + S[i] + K[i % (3 + opt.keylen)]) & 0xFF;
             SWAP( S[i], S[j] );
@@ -1299,7 +1322,7 @@ int check_wep_key( int B )
 
     nb_ascii = 0;
 
-    for( i = 0; i < opt.keylen; i++ )
+    for( i = 0; i < opt.keylen; ++i )
         if(   wep.key[i] == 0 ||
             ( wep.key[i] >= 32 && wep.key[i] < 127 ) )
             nb_ascii++;
@@ -1323,19 +1346,19 @@ int check_wep_key( int B )
         printf( "\33[K\33[%dCKEY FOUND! [ ", n );
     }
 
-    for( i = 0; i < opt.keylen - 1; i++ )
+    for( i = 0; i < opt.keylen - 1; ++i )
         printf( "%02X:", wep.key[i] );
     printf( "%02X ] ",   wep.key[i] );
 
     if( 100 * nb_ascii > 75 * opt.keylen )
     {
-        printf( "(" );
+        printf( "(ASCII: " );
 
-        for( i = 0; i < opt.keylen; i++ )
+        for( i = 0; i < opt.keylen; ++i )
             printf( "%c", ( ( wep.key[i] >  31 && wep.key[i] < 127 ) ||
                               wep.key[i] > 160 ) ? wep.key[i] : '.' );
 
-        printf( ")" );
+        printf( " )" );
     }
 
     if( opt.l33t )
@@ -1368,7 +1391,7 @@ int calc_poll( int B )
 
     /* send the current keybyte # to each thread */
 
-    for( cid = 0; cid < opt.nbcpu; cid++ )
+    for( cid = 0; cid < opt.nbcpu; ++cid )
     {
         n = sizeof( int );
 
@@ -1382,13 +1405,13 @@ int calc_poll( int B )
 
     /* collect the votes, multiply by the korek coeffs */
 
-    for( i = 0; i < 256; i++ )
+    for( i = 0; i < 256; ++i )
     {
         wep.poll[B][i].idx = i;
         wep.poll[B][i].val = 0;
     }
 
-    for( cid = 0; cid < opt.nbcpu; cid++ )
+    for( cid = 0; cid < opt.nbcpu; ++cid )
     {
         n = sizeof( votes );
 
@@ -1399,8 +1422,8 @@ int calc_poll( int B )
             _exit( FAILURE );
         }
 
-        for( n = 0, vi = (int *) votes; n < N_ATTACKS; n++ )
-            for( i = 0; i < 256; i++, vi++ )
+        for( n = 0, vi = (int *) votes; n < N_ATTACKS; ++n )
+            for( i = 0; i < 256; ++i, ++vi )
                 wep.poll[B][i].val += *vi * K_COEFF[n];
     }
 
@@ -1413,10 +1436,10 @@ int calc_poll( int B )
 
     if( opt.is_alnum )
     {
-        for( i = 1; i < 32; i++ )
+        for( i = 1; i < 32; ++i )
             wep.poll[B][i].val = -1;
 
-        for( i = 127; i < 256; i++ )
+        for( i = 127; i < 256; ++i )
             wep.poll[B][i].val = -1;
     }
 
@@ -1424,7 +1447,7 @@ int calc_poll( int B )
 
     if( opt.is_bcdonly )
     {
-        for( i = 1; i < 256; i++ )
+        for( i = 1; i < 256; ++i )
             if( i > 0x99 || ( i & 0x0F ) > 0x09 )
                 wep.poll[B][i].val = -1;
     }
@@ -1561,14 +1584,14 @@ get_ivs:
 
     /* fudge threshold = higest vote divided by fudge factor */
 
-    for( wep.fudge[B] = 1; wep.fudge[B] < 256; wep.fudge[B]++ )
+    for( wep.fudge[B] = 1; wep.fudge[B] < 256; ++( wep.fudge[B] ) )
         if( (float) wep.poll[B][wep.fudge[B]].val <
             (float) wep.poll[B][0].val / opt.ffact )
             break;
 
     /* try the most likely n votes, where n is the fudge threshold */
 
-    for( wep.depth[B] = 0; wep.depth[B] < wep.fudge[B]; wep.depth[B]++ )
+    for( wep.depth[B] = 0; wep.depth[B] < wep.fudge[B]; ++( wep.depth[B] ) )
     {
         switch( update_ivbuf() )
         {
@@ -1600,11 +1623,11 @@ get_ivs:
             /* as noted by Simon Marechal, it's more efficient
              * to just bruteforce the last two keybytes. */
 
-            for( i = 0; i < 256; i++ )
+            for( i = 0; i < 256; ++i )
             {
                 wep.key[B + 1] = i;
 
-                for( j = 0; j < 256; j++ )
+                for( j = 0; j < 256; ++j )
                 {
                     wep.key[B + 2] = j;
 
@@ -1650,7 +1673,7 @@ int do_wep_crack2( int B )
         _exit( FAILURE );
     }
 
-    for( i = 0; i <= B; i++ )
+    for( i = 0; i <= B; ++i )
     {
         if( calc_poll( i ) != SUCCESS )
             return( FAILURE );
@@ -1664,12 +1687,12 @@ int do_wep_crack2( int B )
             show_wep_stats( i, 0 );
     }
 
-    for( wep.fudge[B] = 1; wep.fudge[B] < 256; wep.fudge[B]++ )
+    for( wep.fudge[B] = 1; wep.fudge[B] < 256; ++( wep.fudge[B] ) )
         if( (float) wep.poll[B][wep.fudge[B]].val <
             (float) wep.poll[B][0].val / opt.ffact )
             break;
 
-    for( wep.depth[B] = 0; wep.depth[B] < wep.fudge[B]; wep.depth[B]++ )
+    for( wep.depth[B] = 0; wep.depth[B] < wep.fudge[B]; ++( wep.depth[B] ) )
     {
         switch( update_ivbuf() )
         {
@@ -1683,7 +1706,7 @@ int do_wep_crack2( int B )
         if( ! opt.is_quiet )
             show_wep_stats( B, 0 );
 
-        for( i = B + 1; i < opt.keylen - 2; i++ )
+        for( i = B + 1; i < opt.keylen - 2; ++i )
         {
             if( calc_poll( i ) != SUCCESS )
                 return( FAILURE );
@@ -1697,11 +1720,11 @@ int do_wep_crack2( int B )
                 show_wep_stats( i, 0 );
         }
 
-        for( i = 0; i < 256; i++ )
+        for( i = 0; i < 256; ++i )
         {
             wep.key[opt.keylen - 2] = i;
 
-            for( j = 0; j < 256; j++ )
+            for( j = 0; j < 256; ++j )
             {
                 wep.key[opt.keylen - 1] = j;
 
@@ -1731,13 +1754,13 @@ void calc_pmk( char *key, char *essid, uchar pmk[40] )
     memset( buffer, 0, sizeof( buffer ) );
     strncpy( (char *) buffer, key, sizeof( buffer ) - 1 );
 
-    for( i = 0; i < 64; i++ )
+    for( i = 0; i < 64; ++i )
         buffer[i] ^= 0x36;
 
     sha1_starts( &ctx_ipad );
     sha1_update( &ctx_ipad, buffer, 64 );
 
-    for( i = 0; i < 64; i++ )
+    for( i = 0; i < 64; ++i )
         buffer[i] ^= 0x6A;
 
     sha1_starts( &ctx_opad );
@@ -1750,7 +1773,7 @@ void calc_pmk( char *key, char *essid, uchar pmk[40] )
                (uchar *) essid, slen, pmk );
     memcpy( buffer, pmk, 20 );
 
-    for( i = 1; i < 4096; i++ )
+    for( i = 1; i < 4096; ++i )
     {
         memcpy( &sha1_ctx, &ctx_ipad, sizeof( sha1_ctx ) );
         sha1_update( &sha1_ctx, buffer, 20 );
@@ -1760,7 +1783,7 @@ void calc_pmk( char *key, char *essid, uchar pmk[40] )
         sha1_update( &sha1_ctx, buffer, 20 );
         sha1_finish( &sha1_ctx, buffer );
 
-        for( j = 0; j < 20; j++ )
+        for( j = 0; j < 20; ++j )
             pmk[j] ^= buffer[j];
     }
 
@@ -1769,7 +1792,7 @@ void calc_pmk( char *key, char *essid, uchar pmk[40] )
                (uchar *) essid, slen, pmk + 20 );
     memcpy( buffer, pmk + 20, 20 );
 
-    for( i = 1; i < 4096; i++ )
+    for( i = 1; i < 4096; ++i )
     {
         memcpy( &sha1_ctx, &ctx_ipad, sizeof( sha1_ctx ) );
         sha1_update( &sha1_ctx, buffer, 20 );
@@ -1779,7 +1802,7 @@ void calc_pmk( char *key, char *essid, uchar pmk[40] )
         sha1_update( &sha1_ctx, buffer, 20 );
         sha1_finish( &sha1_ctx, buffer );
 
-        for( j = 0; j < 20; j++ )
+        for( j = 0; j < 20; ++j )
             pmk[j + 20] ^= buffer[j];
     }
 }
@@ -1852,7 +1875,7 @@ int crack_wpa_thread( void *arg )
         v = (uint *) ( k_ipad + 64 );
         w = (uint *) buffer;
 
-        for( i = 0; i < 16; i++ )
+        for( i = 0; i < 16; ++i )
         {
             /* interleave the data */
 
@@ -1867,7 +1890,7 @@ int crack_wpa_thread( void *arg )
         v = (uint *) ( k_opad + 64 );
         w = (uint *) buffer;
 
-        for( i = 0; i < 16; i++ )
+        for( i = 0; i < 16; ++i )
         {
             *w++ = *u++ ^ 0x5C5C5C5C;
             *w++ = *v++ ^ 0x5C5C5C5C;
@@ -1902,7 +1925,7 @@ int crack_wpa_thread( void *arg )
         *w++ = *u++; *w++ = *v++;
         *w++ = *u++; *w++ = *v++;
 
-        for( i = 1; i < 4096; i++ )
+        for( i = 1; i < 4096; ++i )
         {
             memcpy( sha1_ctx, ctx_ipad, 40 );
             shammx_data( sha1_ctx, buffer, wrkbuf );
@@ -1943,7 +1966,7 @@ int crack_wpa_thread( void *arg )
         *w++ = *u++; *w++ = *v++;
         *w++ = *u++; *w++ = *v++;
 
-        for( i = 1; i < 4096; i++ )
+        for( i = 1; i < 4096; ++i )
         {
             memcpy( sha1_ctx, ctx_ipad, 40 );
             shammx_data( sha1_ctx, buffer, wrkbuf );
@@ -2032,7 +2055,7 @@ void show_wpa_stats( char *key, uchar pmk[32], uchar ptk[64],
     printf( "\33[11;7HMaster Key     : " );
 
     if( opt.l33t ) printf( "\33[32;1m" );
-    for( i = 0; i < 32; i++ )
+    for( i = 0; i < 32; ++i )
     {
         if( i == 16 ) printf( "\n\33[23C" );
         printf( "%02X ", pmk[i] );
@@ -2042,7 +2065,7 @@ void show_wpa_stats( char *key, uchar pmk[32], uchar ptk[64],
     printf( "\33[14;7HTranscient Key : " );
 
     if( opt.l33t ) printf( "\33[32;1m" );
-    for( i = 0; i < 64; i++ )
+    for( i = 0; i < 64; ++i )
     {
         if( i > 0 && i % 16 == 0 ) printf( "\n\33[23C" );
         printf( "%02X ", ptk[i] );
@@ -2052,7 +2075,7 @@ void show_wpa_stats( char *key, uchar pmk[32], uchar ptk[64],
     printf( "\33[19;7HEAPOL HMAC     : " );
 
     if( opt.l33t ) printf( "\33[32;1m" );
-    for( i = 0; i < 16; i++ )
+    for( i = 0; i < 16; ++i )
         printf( "%02X ", mic[i] );
 
     printf( "\n" );
@@ -2070,7 +2093,7 @@ int do_wpa_crack( struct AP_info *ap )
 
     /* send the ESSID to each thread */
 
-    for( cid = 0; cid < opt.nbcpu; cid++ )
+    for( cid = 0; cid < opt.nbcpu; ++cid )
     {
         if( safe_write( mc_pipe[cid][1], (void *) ap->essid, 32 ) != 32 )
         {
@@ -2119,13 +2142,12 @@ int do_wpa_crack( struct AP_info *ap )
         if( opt.l33t )
             printf( "\33[34;1m" );
 
-        printf( "\33[2;34Haircrack %d.%d",
-                _MAJ, _MIN );
+        printf("\33[2;34%s",progname);
     }
 
     while( 1 )
     {
-        for( cid = 0; cid < opt.nbcpu; cid++ )
+        for( cid = 0; cid < opt.nbcpu; ++cid )
         {
             /* read a couple of keys (skip those < 8 chars) */
 
@@ -2136,7 +2158,7 @@ int do_wpa_crack( struct AP_info *ap )
                     if( opt.l33t )
                         printf( "\33[32;22m" );
 
-                    printf( "\nEOF\n" );
+                    printf( "\nPassphrase not in dictionnary\n" );
 
                     return( FAILURE );
                 }
@@ -2145,6 +2167,7 @@ int do_wpa_crack( struct AP_info *ap )
 
                 if( key1[i - 1] == '\n' ) key1[--i] = '\0';
                 if( key1[i - 1] == '\r' ) key1[--i] = '\0';
+                
             }
             while( i < 8 );
 
@@ -2157,6 +2180,7 @@ int do_wpa_crack( struct AP_info *ap )
 
                 if( key2[i - 1] == '\n' ) key2[--i] = '\0';
                 if( key2[i - 1] == '\r' ) key2[--i] = '\0';
+                
             }
             while( i < 8 );
 
@@ -2170,7 +2194,7 @@ int do_wpa_crack( struct AP_info *ap )
             }
         }
 
-        for( cid = 0; cid < opt.nbcpu; cid++ )
+        for( cid = 0; cid < opt.nbcpu; ++cid )
         {
             /* collect and test the master keys */
 
@@ -2185,7 +2209,7 @@ int do_wpa_crack( struct AP_info *ap )
 
             /* compute the pairwise transient key and the frame MIC */
 
-            for( i = 0; i < 4; i++ )
+            for( i = 0; i < 4; ++i )
             {
                 pke[99] = i;
                 hmac_sha1( pmk1, 32, pke, 100, ptk1 + i * 20 );
@@ -2263,45 +2287,51 @@ void sighandler( int signum )
         printf( "\33[2J\n" );
 }
 
-
-char * getVersion(char * progname, int maj, int min, int submin)
-{
-	char * temp;
-	temp = (char *) calloc(1,strlen(progname)+50);
-
-	if (submin>0) {
-		sprintf(temp, "%s %d.%d.%d", progname, maj, min, submin);	
-	} else {
-		sprintf(temp, "%s %d.%d", progname, maj, min);
-	}
-	temp = realloc(temp, strlen(temp)+1);
-	return temp;
-}
-
 int main( int argc, char *argv[] )
 {
-    int i, n, ret;
+    int i, n, ret, max_cpu, option;
     char *s, buf[128];
     struct AP_info *ap_cur;
 
     ret = FAILURE;
 
-    /* check the arguments */
-
+	progname = getVersion("Aircrack-ng", _MAJ, _MIN, _SUB_MIN, _BETA);
+	
+	memset( &opt, 0, sizeof( opt ) );
+	
+	#ifdef _SC_NPROCESSORS_ONLN
+	
+		max_cpu   = sysconf(_SC_NPROCESSORS_ONLN);
+		opt.nbcpu = max_cpu;
+	
+	#else
+	
+		max_cpu   = 255;
+		opt.nbcpu =   1;
+	
+	#endif
+	
+	
+	/* check the arguments */
+	
     if( argc < 2 )
     {
     usage:
-        printf( usage, getVersion("Aircrack-ng", _MAJ, _MIN, _SUB_MIN) );
+    	printf (usage, progname, 
+    		( max_cpu == 1 ) ? "\n" : "\n      -p <nbcpu> : # of CPU to use\n" );
         return( ret );
     }
 
-    memset( &opt, 0, sizeof( opt ) );
 
-    opt.nbcpu = 1;
+    opt.do_brute = 1;
 
     while( 1 )
     {
-        int option = getopt( argc, argv, "a:e:b:p:qctd:m:n:i:f:k:xyw:0" );
+
+        if ( max_cpu == 1 )
+        	option = getopt( argc, argv, "a:e:b:qctd:m:n:i:f:k:xyw:0" );
+        else
+        	option = getopt( argc, argv, "a:e:b:p:qctd:m:n:i:f:k:xyw:0" );
 
         if( option < 0 ) break;
 
@@ -2311,6 +2341,12 @@ int main( int argc, char *argv[] )
 
                 sscanf( optarg, "%d", &opt.amode );
 
+				if ( strcasecmp( optarg, "wep" ) == 0 )
+					opt.amode = 1;
+				
+				if ( strcasecmp( optarg, "wpa" ) == 0 )
+					opt.amode = 2;
+
                 if( opt.amode != 1 && opt.amode != 2 )
                 {
                     printf( "Invalid attack mode.\n" );
@@ -2359,16 +2395,15 @@ int main( int argc, char *argv[] )
                 break;
 
             case 'p' :
-
                 if( sscanf( optarg, "%d", &opt.nbcpu ) != 1 ||
-                    opt.nbcpu < 1 || opt.nbcpu > 256 )
+                    opt.nbcpu < 1 || opt.nbcpu > max_cpu )
                 {
-                    printf( "Invalid number of processes.\n" );
-                    return( FAILURE );
+                	printf( "Invalid number of processes.\n" );
+                	return( FAILURE );
                 }
 
-                break;
-
+				break;
+				
             case 'q' :
 
                 opt.is_quiet = 1;
@@ -2402,7 +2437,7 @@ int main( int argc, char *argv[] )
                         return( FAILURE );
                     }
 
-                    opt.debug[i++] = n;
+                    opt.debug[++i] = n;
 
                     if( i >= 62 ) break;
 
@@ -2510,7 +2545,7 @@ int main( int argc, char *argv[] )
 
             case 'x' :
 
-                opt.do_brute = 1;
+                opt.do_brute = 0;
                 break;
 
             case 'y' :
@@ -2519,7 +2554,6 @@ int main( int argc, char *argv[] )
                 break;
 
             case 'w' :
-
                 if( strcmp( optarg, "-" ) == 0 )
                 {
                     if( ( opt.dict = fdopen( 0, "r" ) ) == NULL )
@@ -2537,8 +2571,18 @@ int main( int argc, char *argv[] )
                         perror( "fopen(dictionary) failed" );
                         return( FAILURE );
                     }
+                    
+                    fseek(opt.dict, 0L, SEEK_END);
+                    
+                    if ( ftell( opt.dict ) <= 0L )
+                    {
+                    	fclose( opt.dict );
+                    	printf( "Empty dictionnary\n" );
+                    	return( FAILURE );
+                    }
+                    
+                    rewind( opt.dict );
                 }
-
                 break;
 
             case '0' :
@@ -2553,6 +2597,19 @@ int main( int argc, char *argv[] )
     if( ! ( argc - optind ) )
         goto usage;
 
+    if( opt.amode == 2 && opt.dict == NULL )
+    {
+    	nodict:
+        printf( "Please specify a dictionary (option -w).\n" );
+        goto exit_main;
+    }
+
+    if( (! opt.essid_set && ! opt.bssid_set) && ( opt.is_quiet || opt.no_stdin ) )
+    {
+        printf( "Please specify an ESSID or BSSID.\n" );
+        goto exit_main;
+    }
+
     /* start one thread per input file */
 
     signal( SIGINT,  sighandler );
@@ -2616,12 +2673,6 @@ int main( int argc, char *argv[] )
     {
         /* ask the user which network is to be cracked */
 
-        if( opt.is_quiet || opt.no_stdin )
-        {
-            printf( "Please specify an ESSID or BSSID.\n" );
-            goto exit_main;
-        }
-
         printf( "   #  BSSID%14sESSID%21sEncryption\n\n", "", "" );
 
         i = 1;
@@ -2661,7 +2712,7 @@ int main( int argc, char *argv[] )
                          break;
             }
 
-            i++; ap_cur = ap_cur->next;
+            ++i; ap_cur = ap_cur->next;
         }
 
         printf( "\n" );
@@ -2679,7 +2730,7 @@ int main( int argc, char *argv[] )
 
                 i = 1; ap_cur = ap_1st;
                 while( ap_cur != NULL && i < n )
-                    { i++; ap_cur = ap_cur->next; }
+                    { ++i; ap_cur = ap_cur->next; }
             }
             while( n < 0 || ap_cur == NULL );
         }
@@ -2746,7 +2797,7 @@ int main( int argc, char *argv[] )
 
     /* create the cracker<->master communication pipes */
 
-    for( i = 0; i < opt.nbcpu; i++ )
+    for( i = 0; i < opt.nbcpu; ++i )
     {
         pipe( mc_pipe[i] );
         pipe( cm_pipe[i] );
@@ -2791,7 +2842,7 @@ int main( int argc, char *argv[] )
 
         memset( &wep, 0, sizeof( wep ) );
 
-        for( i = 0; i < opt.nbcpu; i++ )
+        for( i = 0; i < opt.nbcpu; ++i )
         {
             /* start one thread per cpu */
 
@@ -2811,6 +2862,7 @@ int main( int argc, char *argv[] )
             while( ret == RESTART );
 
             if( ret == FAILURE )
+            {
                 printf( "   Attack failed. Possible reasons:\n\n"
 "     * Out of luck: you must capture more IVs. Usually, 104-bit WEP\n"
 "       can be cracked with about one million IVs, sometimes more.\n\n"
@@ -2818,11 +2870,15 @@ int main( int argc, char *argv[] )
 "       then the capture file is corrupted, or the key is not static.\n\n"
 "     * A false positive prevented the key from being found.  Try to\n"
 "       disable each korek attack (-k 1 .. 17), raise the fudge factor\n"
-"       (-f) and try the experimental bruteforce attacks (-x / -y).\n" );
+"       (-f)" );
+                if (opt.do_testy)
+			         printf( "and try the experimental bruteforce attacks (-y)." );
+                printf( "\n" );
+		    }
         }
         else
         {
-            for( i = opt.keylen - 3; i < opt.keylen - 2; i++ )
+            for( i = opt.keylen - 3; i < opt.keylen - 2; ++i )
             {
                 do   { ret = do_wep_crack2( i ); }
                 while( ret == RESTART );
@@ -2832,6 +2888,7 @@ int main( int argc, char *argv[] )
             }
 
             if( ret == FAILURE )
+            {
                 printf( "   Attack failed. Possible reasons:\n\n"
 "     * Out of luck: you must capture more IVs. Usually, 104-bit WEP\n"
 "       can be cracked with about one million IVs, sometimes more.\n\n"
@@ -2839,7 +2896,11 @@ int main( int argc, char *argv[] )
 "       then the capture file is corrupted, or the key is not static.\n\n"
 "     * A false positive prevented the key from being found.  Try to\n"
 "       disable each korek attack (-k 1 .. 17), raise the fudge factor\n"
-"       (-f) or try the standard attack mode instead (no -y option).\n" );
+"       (-f)" );
+                if (opt.do_testy)
+		             printf( "or try the standard attack mode instead (no -y option)." );
+                printf( "\n" );
+            }
         }
     }
 
@@ -2847,11 +2908,8 @@ int main( int argc, char *argv[] )
     {
     crack_wpa:
 
-        if( opt.dict == NULL )
-        {
-            printf( "Please specify a dictionary (option -w).\n" );
-            goto exit_main;
-        }
+		if ( opt.dict == NULL )
+			goto nodict;
 
         ap_cur = ap_1st;
 
@@ -2881,7 +2939,7 @@ int main( int argc, char *argv[] )
             strncpy( ap_cur->essid, opt.essid, sizeof( ap_cur->essid ) - 1 );
         }
 
-        for( i = 0; i < opt.nbcpu; i++ )
+        for( i = 0; i < opt.nbcpu; ++i )
         {
             /* start one thread per cpu */
 
