@@ -2,7 +2,7 @@
  *  802.11 WEP replay & injection attacks
  *
  *  Copyright (C) 2006 Thomas d'Otreppe
- *  Original work: Christophe Devine
+ *  Copyright (C) 2004,2005  Christophe Devine
  *
  *  WEP decryption attack (chopchop) developped by KoreK
  *
@@ -39,6 +39,7 @@
 #include <fcntl.h>
 #include <errno.h>
 #include <time.h>
+#include <getopt.h>
 
 #include "version.h"
 #include "pcap.h"
@@ -74,8 +75,18 @@
 #define RATES           \
     "\x01\x04\x02\x04\x0B\x16\x32\x08\x0C\x12\x18\x24\x30\x48\x60\x6C"
 
+extern char * getVersion(char * progname, int maj, int min, int submin, int betavers);
+extern int is_ndiswrapper(const char * iface);
+extern int iwpriv_exist(void);
+
 char usage[] =
 
+"\n"
+"  %s - (C) 2006 Thomas d\'Otreppe\n"
+"  Original work: Christophe Devine\n"
+"  http://www.aircrack-ng.org\n"
+"\n"
+"  usage: aireplay-ng <options> <replay interface>\n"
 "\n"
 "  filter options:\n"
 "\n"
@@ -86,8 +97,8 @@ char usage[] =
 "      -n len    : maximum packet length\n"
 "      -u type   : frame control, type    field\n"
 "      -v subt   : frame control, subtype field\n"
-"      -t tods   : frame control, To   DS bit\n"
-"      -f fromds : frame control, From DS bit\n"
+"      -t tods   : frame control, To      DS bit\n"
+"      -f fromds : frame control, From    DS bit\n"
 "      -w iswep  : frame control, WEP     bit\n"
 "\n"
 "  replay options:\n"
@@ -97,8 +108,8 @@ char usage[] =
 "      -a bssid  : set Access Point MAC address\n"
 "      -c dmac   : set Destination  MAC address\n"
 "      -h smac   : set Source       MAC address\n"
-"      -e essid  : attack 1: set target AP SSID\n"
-"      -j essid  : attack 3: inject FromDS pkts\n"
+"      -e essid  : fakeauth  attack : set target AP SSID\n"
+"      -j        : arpreplay attack : inject FromDS pkts\n"
 "\n"
 "  source options:\n"
 "\n"
@@ -107,17 +118,11 @@ char usage[] =
 "\n"
 "  attack modes:\n"
 "\n"
-"      -0 count  : deauthenticate all stations\n"
-"      -1 delay  : fake authentication with AP\n"
-"      -2        : interactive frame selection\n"
-"      -3        : standard ARP-request replay\n"
-"      -4        : decrypt/chopchop WEP packet\n"
-"\n"
-"  %s - (C) 2006 Thomas d\'Otreppe\n"
-"  Original work: Christophe Devine\n"
-"  http://www.aircrack-ng.org\n"
-"\n"
-"  usage: aireplay-ng [options] <replay interface>\n"
+"      --deauth      count : deauthenticate all stations\n"
+"      --fakeauth    delay : fake authentication with AP\n"
+"      --interactive       : interactive frame selection\n"
+"      --arpreplay         : standard ARP-request replay\n"
+"      --chopchop          : decrypt/chopchop WEP packet\n"
 "\n";
 
 struct options
@@ -159,6 +164,7 @@ struct devices
     int is_wlanng;
     int is_hostap;
     int is_madwifi;
+    int is_madwifing;
 
     FILE *f_cap_in;
 
@@ -290,7 +296,7 @@ int read_packet( void *buf, size_t count )
         return( -1 );
     }
 
-    if( dev.is_madwifi )
+    if( dev.is_madwifi && !(dev.is_madwifing) )
         caplen -= 4;    /* remove the FCS */
 
     memset( buf, 0, sizeof( buf ) );
@@ -1144,7 +1150,8 @@ read_packets:
 
     /* loop resending the packet */
 
-    printf( "You must also start airodump to capture replies.\n\n" );
+	/* Check if airodump-ng is running. If not, print that message */
+    printf( "You should also start airodump-ng to capture replies.\n\n" );
 
     signal( SIGINT, sighandler );
     ctrl_c = 0;
@@ -1287,7 +1294,7 @@ int do_attack_arp_resend( void )
 
     fflush( f_cap_out );
 
-    printf( "You must also start airodump to capture replies.\n" );
+    printf( "You should also start airodump-ng to capture replies.\n" );
 
     /* avoid blocking on reading the socket */
 
@@ -1721,13 +1728,13 @@ int do_attack_chopchop( void )
             if( is_deauth_mode )
                 printf(
 "    * The AP isn't vulnerable when operating in non-authenticated mode.\n"
-"      Run aireplay in authenticated mode instead (-h option).\n\n" );
+"      Run aireplay-ng in authenticated mode instead (-h option).\n\n" );
             else
                 printf(
 "    * The client MAC you have specified is not currently authenticated.\n"
-"      Try running another aireplay to fake authentication (attack \"-1\").\n"
+"      Try running another aireplay-ng to fake authentication (attack \"-1\").\n"
 "    * The AP isn't vulnerable when operating in authenticated mode.\n"
-"      Try aireplay in non-authenticated mode instead (no -h option).\n\n" );
+"      Try aireplay-ng in non-authenticated mode instead (no -h option).\n\n" );
             return( 1 );
         }
 
@@ -1913,7 +1920,7 @@ int do_attack_chopchop( void )
                 {
                     printf(
                 "\n\nFailure: got several deauthentication packets "
-                "from the AP - try running\nanother aireplay with "
+                "from the AP - try running\nanother aireplay-ng with "
                 "attack \"-1\" (fake open-system authentication).\n\n" );
                     return( 1 );
                 }
@@ -1933,7 +1940,7 @@ int do_attack_chopchop( void )
             {
                 printf( "\n\nFailure: the access point does not properly "
                         "discard frames with an\ninvalid ICV - try running "
-                        "aireplay in authenticated mode (-h) instead.\n\n" );
+                        "aireplay-ng in authenticated mode (-h) instead.\n\n" );
                 return( 1 );
             }
         }
@@ -1969,7 +1976,7 @@ int do_attack_chopchop( void )
             {
                 printf( "\n\nFailure: the access point does not properly "
                         "discard frames with an\ninvalid ICV - try running "
-                        "aireplay in non-authenticated mode instead.\n\n" );
+                        "aireplay-ng in non-authenticated mode instead.\n\n" );
                 return( 1 );
             }
         }
@@ -2240,30 +2247,10 @@ int getmac( char *s, unsigned char *mac )
 
 char athXraw[] = "athXraw";
 
-char * getVersion(char * progname, int maj, int min, int submin)
-{
-	char * temp;
-	temp = (char *) calloc(1,strlen(progname)+50);
-
-	if (submin>0) {
-		sprintf(temp, "%s %d.%d.%d", progname, maj, min, submin);	
-	} else {
-		sprintf(temp, "%s %d.%d", progname, maj, min);
-	}
-	temp = realloc(temp, strlen(temp)+1);
-	return temp;
-}
-
 int main( int argc, char *argv[] )
 {
     int n;
 
-    if( geteuid() != 0 )
-    {
-        printf( "This program requires root privileges.\n" );
-        return( 1 );
-    }
-
     /* check the arguments */
 
     memset( &opt, 0, sizeof( opt ) );
@@ -2277,13 +2264,29 @@ int main( int argc, char *argv[] )
 
     while( 1 )
     {
-        int option = getopt( argc, argv,
-                        "b:d:s:m:n:u:v:t:f:w:x:p:a:c:h:e:j:i:r:0:1:234" );
+        int option_index = 0;
+        
+        static struct option long_options[] = {
+            {"deauth",      1, 0, '0'},
+            {"fakeauth",    1, 0, '1'},
+            {"interactive", 0, 0, '2'},
+            {"arpreplay",   0, 0, '3'},
+            {"chopchop",    0, 0, '4'},
+            {0,             0, 0,  0 }
+        };
+
+        int option = getopt_long( argc, argv,
+                        "b:d:s:m:n:u:v:t:f:w:x:p:a:c:h:e:j:i:r:0:1:234",
+                        long_options, &option_index );
 
         if( option < 0 ) break;
 
         switch( option )
         {
+        	case 0 :
+        		
+        		break;
+        		
             case 'b' :
 
                 if( getmac( optarg, opt.f_bssid ) != 0 )
@@ -2530,7 +2533,7 @@ int main( int argc, char *argv[] )
     if( argc - optind < 1 || argc - optind > 2 )
     {
     usage:
-        printf( usage, getVersion("Aireplay-ng", _MAJ, _MIN, _SUB_MIN)  );
+        printf( usage, getVersion("Aireplay-ng", _MAJ, _MIN, _SUB_MIN, _BETA)  );
         return( 1 );
     }
 
@@ -2547,6 +2550,12 @@ int main( int argc, char *argv[] )
         return( 1 );
     }
 
+    if( geteuid() != 0 )
+    {
+        printf( "This program requires root privileges.\n" );
+        return( 1 );
+    }
+
     memset( &dev, 0, sizeof( dev ) );
 
     dev.fd_rtc = -1;
@@ -2595,6 +2604,22 @@ int main( int argc, char *argv[] )
         return( 1 );
     }
 
+	/* Check iwpriv existence */
+	
+    if (! iwpriv_exist() )
+	{
+		printf("Can't find wireless tools, exiting.\n");
+		return (1);
+	}
+    
+	/* Exit if ndiswrapper : check iwpriv ndis_reset */
+	
+	if ( is_ndiswrapper(argv[optind]) )
+	{
+		printf("Ndiswrapper doesn't support monitor mode.\n");
+		return (1);
+	}
+
     if( ( dev.fd_out = socket( PF_PACKET, SOCK_RAW,
                                htons( ETH_P_ALL ) ) ) < 0 )
     {
@@ -2645,6 +2670,7 @@ int main( int argc, char *argv[] )
     if( strlen( argv[optind] ) == 4 &&
         memcmp( argv[optind], "ath", 3 ) == 0 )
     {
+    	dev.is_madwifi=1;
         memset( strbuf, 0, sizeof( strbuf ) );
         snprintf( strbuf,  sizeof( strbuf ) - 1,
                   "sysctl -w dev.%s.rawdev=1 &>/dev/null",
@@ -2652,6 +2678,7 @@ int main( int argc, char *argv[] )
 
         if( system( strbuf ) == 0 )
         {
+        	
             athXraw[3] = argv[optind][3];
 
             memset( strbuf, 0, sizeof( strbuf ) );
@@ -2669,8 +2696,13 @@ int main( int argc, char *argv[] )
 
             argv[optind] = athXraw;
         }
+        else 
+        {
+        	// It is madwifi-ng
+        	dev.is_madwifing=1;
+        }
     }
-
+	
     /* drop privileges */
 
     setuid( getuid() );
