@@ -2,7 +2,7 @@
  *  pcap-compatible 802.11 packet sniffer
  *
  *  Copyright (C) 2006 Thomas d'Otreppe
- *  Original work: Christophe Devine
+ *  Copyright (C) 2004,2005  Christophe Devine
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -36,6 +36,13 @@
 #include <stdio.h>
 #include <errno.h>
 #include <time.h>
+#include <getopt.h>
+
+#ifdef linux
+    int linux_acpi;
+    int linux_apm;
+    #include <dirent.h>
+#endif
 
 #include "version.h"
 #include "pcap.h"
@@ -57,8 +64,31 @@
 #define NB_PWR  5       /* size of signal power ring buffer */
 #define NB_PRB 10       /* size of probed ESSID ring buffer */
 
+extern char * getVersion(char * progname, int maj, int min, int submin, int betavers);
+extern int is_ndiswrapper(const char * iface);
+extern int iwpriv_exist(void);
+
 char *f_ext[4] = { "txt", "gps", "cap", "ivs" };
-int bg_chans[] = { 1, 7, 13, 2, 8, 3, 14, 9, 4, 10, 5, 11, 6, 12, 0 };
+
+int abg_chans [] = 
+{
+    1, 7, 13, 2, 8, 3, 14, 9, 4, 10, 5, 11, 6, 12,
+    36, 40, 44, 48, 52, 56, 60, 64, 100, 104, 108,
+    112, 116, 120, 124, 128, 132, 136, 140, 149,
+    153,157,161,184,188,192,196,200,204,208,212,216,0
+};
+
+int bg_chans  [] =
+{
+    1, 7, 13, 2, 8, 3, 14, 9, 4, 10, 5, 11, 6, 12, 0
+};
+
+int a_chans   [] =
+{
+    36, 40, 44, 48, 52, 56, 60, 64, 100, 104, 108,
+    112, 116, 120, 124, 128, 132, 136, 140, 149,
+    153,157,161,184,188,192,196,200,204,208,212,216,0
+};
 
 /* linked list of detected access points */
 
@@ -121,17 +151,22 @@ struct globals
     FILE *f_cap;            /* output cap file      */
     FILE *f_ivs;            /* output ivs file      */
 
-    int battery;            /* remaining battery %  */
+    char * batt;            /* Battery string       */
     int channel;            /* current channel #    */
     int ch_pipe[2];         /* current channel pipe */
     int gc_pipe[2];         /* gps coordinates pipe */
     float gps_loc[5];       /* gps coordinates      */
     int save_gps;           /* keep gps file flag   */
+    int usegpsd;            /* do we use GPSd?      */
+    int * channels;
 
     int is_wlanng;          /* set if wlan-ng       */
     int is_orinoco;         /* set if orinoco       */
+    int is_madwifing;       /* set if madwifi-ng    */
     int do_exit;            /* interrupt flag       */
     struct winsize ws;      /* console window size  */
+    
+    char * elapsed_time;	/* capture time			*/
 }
 G;
 
@@ -143,14 +178,21 @@ char usage[] =
 "  Original work: Christophe Devine\n"
 "  http://www.aircrack-ng.org\n"
 "\n"
-"  usage: airodump-ng <interface> <output prefix> [channel] [IVs flag]\n"
+"  usage: airodump-ng <options> <interface>\n"
 "\n"
-"  Specify 0 as the channel to hop between 2.4 GHz channels.\n"
-"  Set the optional IVs flag to 1 to only save the captured\n"
-"  IVs - the resulting file is only useful for WEP cracking.\n"
+"  Options:\n"
+"      --ivs               : Save only captured IVs\n"
+"      --gpsd              : Use GPSd\n"
+"      --write    <prefix> : Dump file prefix (mandatory)\n"
+"      -w                  : same as --write \n"
 "\n"
-"  If the gpsd daemon is running, airodump will retrieve and\n"
-"  save the current GPS coordinates in text format.\n"
+"  By default, airodump-ng hop on all available channels.\n"
+"  You can make it hop on bg channels only or a channels only:\n"
+"      --channel <channel> : Capture on a specific channel\n"
+"      -c                  : Same as --channel\n"
+"      --bg                : Will hop on 2.4Ghz channels\n"
+"      -b                  : same as --bg\n"
+"      -a                  : Will hop on 5Ghz channels\n"
 "\n";
 
 
@@ -190,7 +232,7 @@ int dump_initialize( char *prefix, int ivs_only )
     }
     while( i < 4 );
 
-    /* create the output CVS & GPS files */
+    /* create the output CSV & GPS files */
 
     snprintf( ofn,  sizeof( ofn ) - 1, "%s-%02d.txt",
               prefix, G.f_index );
@@ -202,16 +244,18 @@ int dump_initialize( char *prefix, int ivs_only )
         return( 1 );
     }
 
-    snprintf( ofn,  sizeof( ofn ) - 1, "%s-%02d.gps",
-              prefix, G.f_index );
-
-    if( ( G.f_gps = fopen( ofn, "wb+" ) ) == NULL )
+    if (G.usegpsd)
     {
-        perror( "fopen failed" );
-        fprintf( stderr, "Could not create \"%s\".\n", ofn );
-        return( 1 );
-    }
+        snprintf( ofn,  sizeof( ofn ) - 1, "%s-%02d.gps",
+                  prefix, G.f_index );
 
+        if( ( G.f_gps = fopen( ofn, "wb+" ) ) == NULL )
+        {
+            perror( "fopen failed" );
+            fprintf( stderr, "Could not create \"%s\".\n", ofn );
+            return( 1 );
+        }
+    }
     /* create the output packet capture file */
 
     if( ivs_only == 0 )
@@ -859,6 +903,229 @@ void dump_sort_power( void )
     G.st_end = new_st_end;
 }
 
+int getBatteryState()
+{
+#ifdef linux
+    char buf[128];
+    int batteryTime = 0;
+    FILE *apm;
+    int flag;
+    char units[32];
+
+    if (linux_apm == 1) 
+    {
+        if ((apm = fopen("/proc/apm", "r")) != NULL) {
+            int charging, ac;
+            fgets(buf, 128, apm);
+            fclose(apm);
+
+            sscanf(buf, "%*s %*d.%*d %*x %x %x %x %*d%% %d %s\n", &ac,
+                &charging, &flag, &batteryTime, units);
+
+            if ((flag & 0x80) == 0 && charging != 0xFF && ac != 1 && batteryTime != -1) {
+                if (!strncmp(units, "min", 32))
+                    batteryTime *= 60;
+            }
+            else return 0;
+            linux_acpi = 0;
+            return batteryTime;
+        }
+        linux_apm = 0;
+    }
+    if (linux_acpi && !linux_apm)
+    {
+        DIR *batteries, *ac_adapters;
+        struct dirent *this_battery, *this_adapter;
+        FILE *acpi, *info;
+        char battery_state[128];
+        char battery_info[128];
+        int rate = 1, remain = 0, current = 0;
+        static int total_remain = 0, total_cap = 0;
+        int batno = 0;
+        static int info_timer = 0;
+        int batt_full_capacity[3];
+        linux_apm=0;
+        linux_acpi=1;
+        ac_adapters = opendir("/proc/acpi/ac_adapter");
+        if ( ac_adapters == NULL )
+            return 0;
+
+        while (ac_adapters != NULL && ((this_adapter = readdir(ac_adapters)) != NULL)) {
+            if (this_adapter->d_name[0] == '.')
+                continue;
+            /* safe overloaded use of battery_state path var */
+            snprintf(battery_state, sizeof(battery_state),
+                "/proc/acpi/ac_adapter/%s/state", this_adapter->d_name);
+            if ((acpi = fopen(battery_state, "r")) == NULL)
+                continue;
+            if (acpi != NULL) {
+                while(fgets(buf, 128, acpi)) {
+                    if (strstr(buf, "on-line") != NULL) {
+                        fclose(acpi);
+                        if (ac_adapters != NULL)
+                            closedir(ac_adapters);
+                        return 0;
+                    }
+                }
+                fclose(acpi);
+            }
+        }
+
+        if (ac_adapters != NULL)
+            closedir(ac_adapters);
+
+        batteries = opendir("/proc/acpi/battery");
+
+        if (batteries == NULL) {
+            closedir(batteries);
+            return 0;
+        }
+
+        while (batteries != NULL && ((this_battery = readdir(batteries)) != NULL)) {
+            if (this_battery->d_name[0] == '.')
+                continue;
+            
+            snprintf(battery_info, sizeof(battery_info), "/proc/acpi/battery/%s/info", this_battery->d_name);
+            info = fopen(battery_info, "r");    
+            batt_full_capacity[batno] = 0;
+            if ( info != NULL ) {
+                while (fgets(buf, sizeof(buf), info) != NULL)
+                    if (sscanf(buf, "last full capacity:      %d mWh", &batt_full_capacity[batno]) == 1)
+                        continue;
+                fclose(info);
+            }
+    
+                
+            snprintf(battery_state, sizeof(battery_state),
+                "/proc/acpi/battery/%s/state", this_battery->d_name);
+            if ((acpi = fopen(battery_state, "r")) == NULL)
+                continue;
+            while (fgets(buf, 128, acpi)) {
+                if (strncmp(buf, "present:", 8 ) == 0) {
+				/* No information for this battery */
+                    if (strstr(buf, "no" ))
+                        continue;
+                }
+                else if (strncmp(buf, "charging state:", 15) == 0) {
+				/* the space makes it different than discharging */
+                    if (strstr(buf, " charging" )) {
+                        fclose( acpi );
+                        return 0;
+                    }
+                }
+                else if (strncmp(buf, "present rate:", 13) == 0)
+                    rate = atoi(buf + 25);
+                else if (strncmp(buf, "remaining capacity:", 19) == 0) {
+                    remain = atoi(buf + 25);
+                    total_remain += remain;
+                }
+                else if (strncmp(buf, "present voltage:", 17) == 0)
+                    current = atoi(buf + 25);
+            }
+            total_cap += batt_full_capacity[batno];
+            fclose(acpi);
+            batteryTime += (int) (( ((float)remain) /rate ) * 3600);
+            batno++;
+        }
+        info_timer++;
+
+        if (batteries != NULL)
+            closedir(batteries);
+    }
+    return batteryTime;
+#elif defined(_BSD_SOURCE)
+    struct apm_power_info api;
+    int apmfd;
+    if ((apmfd = open("/dev/apm", O_RDONLY)) < 0)
+        return 0;
+    if (ioctl(apmfd, APM_IOC_GETPOWER, &api) < 0) {
+        close(apmfd);
+        return 0;
+    }
+    close(apmfd);
+    if (api.battery_state == APM_BATT_UNKNOWN ||
+        api.battery_state == APM_BATTERY_ABSENT ||
+        api.battery_state == APM_BATT_CHARGING ||
+    api.ac_state == APM_AC_ON) {
+        return 0;
+    }
+    return (int)(api.minutes_left);
+#else
+    return 0;
+#endif
+}
+
+
+char * getStringTimeFromSec(double seconds)
+{
+	int hour[3];
+	char * ret;
+    char * HourTime;
+    char * MinTime;
+
+	if (seconds <0) 
+		return NULL;
+
+	ret = (char *) calloc(1,256);
+
+    HourTime = (char *) calloc (1,128);
+    MinTime  = (char *) calloc (1,128);
+
+	hour[0]  = (int) (seconds);
+	hour[1]  = hour[0] / 60;
+    hour[2]  = hour[1] / 60;
+	hour[0] %= 60 ;
+	hour[1] %= 60 ;
+
+	if (hour[2] != 0 )
+		sprintf(HourTime,"%d %s", hour[2], ( hour[2] == 1 ) ? "hour" : "hours");
+	if (hour[1] != 0 )
+		sprintf(MinTime,"%d %s", hour[1], ( hour[1] == 1 ) ? "min" : "mins");
+
+    if ( hour[2] != 0 && hour[1] != 0 )
+        sprintf(ret, "%s %s", HourTime, MinTime);
+    else
+    {
+    	if (hour[2] == 0 && hour[1] == 0)
+    		sprintf(ret, "%d s", hour[0] );
+        else 
+        	sprintf(ret, "%s", (hour[2] == 0) ? MinTime : HourTime );
+	}
+	
+    free(MinTime);
+    free(HourTime);
+			
+	return ret;
+
+}
+
+
+char * getBatteryString(void)
+{
+    int batt_time;
+    char * ret;
+	char * batt_string;
+
+    batt_time = getBatteryState();
+
+    if ( batt_time <= 60 ) {
+        ret = (char *) calloc(1,2);
+        ret[0] = ']';
+        return ret;
+    }
+
+	batt_string = getStringTimeFromSec( (double) batt_time );
+	
+	ret = (char *) calloc( 1, 256 );
+
+	sprintf( ret,"][ BAT: %s ]", batt_string );
+
+	free( batt_string);
+
+    return ret;
+}
+
+
 void dump_print( int ws_row, int ws_col )
 {
     time_t tt;
@@ -877,19 +1144,31 @@ void dump_print( int ws_row, int ws_col )
     tt = time( NULL );
     lt = localtime( &tt );
 
-    /* display the channel, battery, position and current time */
+    /* 
+     *  display the channel, battery, position (if we are connected to GPSd) 
+     *  and current time
+     */
 
     memset( strbuf, ' ', ws_col - 1 );
     strbuf[ws_col - 1] = '\0';
     fprintf( stderr, "%s\n", strbuf );
+    
 
-    snprintf( strbuf, sizeof( strbuf ) - 1,
-              " CH %2d ][ BAT %3d%% ][ GPS %8.3f %8.3f %8.3f %6.2f "
-              "][ %04d-%02d-%02d %02d:%02d ", G.channel, G.battery,
+    if (G.gps_loc[0]) {
+        snprintf( strbuf, sizeof( strbuf ) - 1,
+              " CH %2d %s[ GPS %8.3f %8.3f %8.3f %6.2f "
+              "][ Elapsed: %s ][ %04d-%02d-%02d %02d:%02d ", G.channel, G.batt,
               G.gps_loc[0], G.gps_loc[1], G.gps_loc[2], G.gps_loc[3],
-              1900 + lt->tm_year, 1 + lt->tm_mon, lt->tm_mday,
-              lt->tm_hour, lt->tm_min );
-
+              G.elapsed_time , 1900 + lt->tm_year,
+              1 + lt->tm_mon, lt->tm_mday, lt->tm_hour, lt->tm_min );
+    }
+    else 
+    {
+        snprintf( strbuf, sizeof( strbuf ) - 1,
+              " CH %2d %s[ Elapsed: %s ][ %04d-%02d-%02d %02d:%02d ",
+              G.channel, G.batt, G.elapsed_time, 1900 + lt->tm_year,
+              1 + lt->tm_mon, lt->tm_mday, lt->tm_hour, lt->tm_min );
+    }
     strbuf[ws_col - 1] = '\0';
     fprintf( stderr, "%s\n", strbuf );
 
@@ -905,7 +1184,7 @@ void dump_print( int ws_row, int ws_col )
     fprintf( stderr, "%s\n", strbuf );
 
     memcpy( strbuf, " BSSID              PWR  Beacons"
-                    "    # IV   CH  MB  ENC   ESSID", 62 );
+                    "   # Data  CH  MB  ENC   ESSID", 62 );
     strbuf[ws_col - 1] = '\0';
     fprintf( stderr, "%s\n", strbuf );
 
@@ -1082,7 +1361,7 @@ void dump_write_csv( void )
 
     fprintf( G.f_txt,
         "\r\nBSSID, First time seen, Last time seen, channel, Speed, "
-        "Privacy, Power, # beacons, # data, LAN IP, ESSID\r\n" );
+        "Privacy, Power, # beacons, # IV, LAN IP, ESSID\r\n" );
 
     ap_cur = G.ap_1st;
 
@@ -1218,99 +1497,29 @@ void dump_write_csv( void )
     sync();
 }
 
-void update_battery( void )
-{
-    FILE *f;
-    char line[256], *p;
-    int a, total, b, remaining, i;
-
-    G.battery = -1;
-
-    if( ( f = fopen( "/proc/apm", "r" ) ) != NULL )
-    {
-        /* old fashioned APM interface */
-
-        memset( line, 0, sizeof( line ) );
-        fgets( line, sizeof( line ) - 1, f );
-        p = strtok( line, " " );
-        for( i = 0; i < 6; i++ )
-        {
-            if( p == NULL ) return;
-            p = strtok( NULL, " " );
-        }
-        if( p == NULL ) return;
-        sscanf( p, "%d%%", &G.battery );
-        return;
-    }
-
-    total = 1;
-    remaining = 0;
-
-    for( i = 0; i < 4; i++ )
-    {
-        /* ACPI interface w/ multiple batteries support */
-
-        memset( line, 0, sizeof( line ) );
-        snprintf( line,  sizeof( line ) - 1,
-                  "/proc/acpi/battery/BAT%d/info", i );
-
-        if( ( f = fopen( line, "r" ) ) == NULL )
-            continue;
-
-        while( fgets( line, sizeof( line ), f ) != NULL )
-        {
-            if( memcmp( line, "design capacity:         ", 25 ) == 0 )
-            {
-                sscanf( line + 25, "%d", &a );
-                break;
-            }   
-        }
-        fclose( f );
-
-        memset( line, 0, sizeof( line ) );
-        snprintf( line,  sizeof( line ) - 1,
-                  "/proc/acpi/battery/BAT%d/state", i );
-
-        if( ( f = fopen( line, "r" ) ) == NULL )
-            continue;
-
-        while( fgets( line, sizeof( line ), f ) != NULL )
-        {
-            if( memcmp( line, "remaining capacity:      ", 25 ) == 0 )
-            {
-                sscanf( line + 25, "%d", &b );
-                break;
-            }   
-        }
-        fclose( f );
-
-        total += a;
-        remaining += b;
-    }
-
-    G.battery = ( 100 * remaining ) / total;
-}
 
 void gps_tracker( void )
 {
     int gpsd_sock;
     char line[256], *p;
     struct sockaddr_in gpsd_addr;
-
+    
     /* attempt to connect to localhost, port 2947 */
 
     gpsd_sock = socket( AF_INET, SOCK_STREAM, 0 );
 
-    if( gpsd_sock < 0 )
+    if( gpsd_sock < 0 ) {
         return;
-
+    }
+    
     gpsd_addr.sin_family      = AF_INET;
     gpsd_addr.sin_port        = htons( 2947 );
     gpsd_addr.sin_addr.s_addr = inet_addr( "127.0.0.1" );
 
     if( connect( gpsd_sock, (struct sockaddr *) &gpsd_addr,
-                 sizeof( gpsd_addr ) ) < 0 )
+                 sizeof( gpsd_addr ) ) < 0 ) {
         return;
+    }
 
     /* loop reading the GPS coordinates */
 
@@ -1324,7 +1533,7 @@ void gps_tracker( void )
 
         memset( line, 0, sizeof( line ) );
         snprintf( line,  sizeof( line ) - 1, "PVTAD\r\n" );
-        if( send( gpsd_sock, line, 3, 0 ) != 3 )
+        if( send( gpsd_sock, line, 7, 0 ) != 7 )
             return;
 
         memset( line, 0, sizeof( line ) );
@@ -1420,9 +1629,10 @@ int set_channel( char *interface, int fd_raw, int channel )
     int pid, status;
     struct iwreq wrq;
 
+    memset( s, 0, sizeof( s ) );
+
     if( G.is_wlanng )
     {
-        memset( s, 0, sizeof( s ) );
         snprintf( s,  sizeof( s ) - 1, "channel=%d", channel );
 
         if( ( pid = fork() ) == 0 )
@@ -1443,7 +1653,6 @@ int set_channel( char *interface, int fd_raw, int channel )
 
     if( G.is_orinoco )
     {
-        memset( s, 0, sizeof( s ) );
         snprintf( s,  sizeof( s ) - 1, "%d", channel );
 
         if( ( pid = fork() ) == 0 )
@@ -1497,8 +1706,7 @@ int set_monitor( char *interface, int fd_raw )
 
         if( WIFEXITED(status) )
             return( WEXITSTATUS(status) );
-        else
-            return( 1 );
+        return( 1 );
     }
 
     memset( &wrq, 0, sizeof( struct iwreq ) );
@@ -1518,22 +1726,20 @@ int set_monitor( char *interface, int fd_raw )
 
 void channel_hopper( char *interface, int fd_raw )
 {
+
     int ch, ch_idx = 0;
 
     while( getppid() != 1 )
     {
-        ch_idx++;
+        ++ch_idx;
 
-        if( bg_chans[ch_idx] ==  0 )
+        if( G.channels[ch_idx] ==  0 )
             ch_idx = 0;
 
-        if( bg_chans[ch_idx] == -1 ) 
-        {
-            usleep( 10000 );
+        if( G.channels[ch_idx] == -1 ) 
             continue;
-        }
 
-        ch = bg_chans[ch_idx];
+        ch = G.channels[ch_idx];
 
         if( set_channel( interface, fd_raw, ch ) == 0 )
         {
@@ -1542,70 +1748,188 @@ void channel_hopper( char *interface, int fd_raw )
             usleep( 350000 );
         }
         else
-            bg_chans[ch_idx] = -1;      /* remove invalid channel */
+            G.channels[ch_idx] = -1;      /* remove invalid channel */
     }
 
     exit( 0 );
 }
 
-char * getVersion(char * progname, int maj, int min, int submin)
-{
-	char * temp;
-	temp = (char *) calloc(1,strlen(progname)+50);
-
-	if (submin>0) {
-		sprintf(temp, "%s %d.%d.%d", progname, maj, min, submin);	
-	} else {
-		sprintf(temp, "%s %d.%d", progname, maj, min);
-	}
-	temp = realloc(temp, strlen(temp)+1);
-	return temp;
-}
-
 int main( int argc, char *argv[] )
 {
     long time_slept;
     int n, caplen, pid;
     int fd_raw, arptype;
     int ivs_only, power;
-    time_t tt1, tt2, tt3;
+    int valid_channel;
+    time_t tt1, tt2, tt3, start_time;
 
-    unsigned char *buffer;
-    unsigned char *h80211;
+    unsigned char      *buffer;
+    unsigned char      *h80211;
+    char               *dump_prefix;
+    char               *iface;
 
-    struct ifreq ifr;
+    struct ifreq       ifr;
     struct packet_mreq mr;
     struct sockaddr_ll sll;
-    struct timeval tv0;
-    struct timeval tv1;
-    struct timeval tv2;
-    struct winsize ws;
-
-    fd_set rfds;
+    struct timeval     tv0;
+    struct timeval     tv1;
+    struct timeval     tv2;
+    struct winsize     ws;
+    /*
+    struct sockaddr_in provis_addr;
+    */
+    
+    fd_set             rfds;
+    FILE               *check_madwifing;
 
     /* initialize a bunch of variables */
 
     memset( &G, 0, sizeof( G ) );
 
-    ivs_only    =  0;
-    power       = -1;
-    fd_raw      = -1;
-    arptype     =  0;
-    time_slept  =  0;
+    ivs_only       =  0;
+    power          = -1;
+    fd_raw         = -1;
+    arptype        =  0;
+    time_slept     =  0;
+    G.batt         =  NULL;
+    valid_channel  =  0;
+    G.usegpsd      =  0;
+    G.channels     =  abg_chans;
+    G.channel      =  0;
+    dump_prefix    =  NULL;
+    #ifdef linux
+        linux_acpi =  1;
+        linux_apm  =  1;
+    #endif
 
     /* check the arguments */
 
-    if( argc < 3 || argc > 5 )
+    if( argc < 4 )
     {
-        printf( usage, getVersion("Airodump-ng", _MAJ, _MIN, _SUB_MIN)  );
+        usage:
+        printf( usage, getVersion("Airodump-ng", _MAJ, _MIN, _SUB_MIN, _BETA)  );
         return( 1 );
     }
 
-    if( argc > 3 )
-        G.channel = atoi( argv[3] );
 
-    if( argc > 4 )
-        ivs_only  = atoi( argv[4] );
+    do
+    {
+        int option_index = 0;
+        
+        static struct option long_options[] = {
+            {"ivs",     0, 0, 'i'},
+            {"channel", 1, 0, 'c'},
+            {"write",   1, 0, 'w'},
+            {"gpsd",    0, 0, 'p'},
+            {"ab",      0, 0, 'h'},
+            {"ag",      0, 0, 'h'},
+            {"ba",      0, 0, 'h'},
+            {"bg",      0, 0, 'b'},
+            {"ga",      0, 0, 'h'},
+            {"gb",      0, 0, 'b'},
+            {"abg",     0, 0, 'h'},
+            {"agb",     0, 0, 'h'},
+            {"bga",     0, 0, 'h'},
+            {"bag",     0, 0, 'h'},
+            {"gba",     0, 0, 'h'},
+            {"gab",     0, 0, 'h'},
+            {"hop",     0, 0, 'h'},
+            {0,         0, 0,  0 }
+        };
+
+        int option = getopt_long( argc, argv,
+                        "abc:giw:",
+                        long_options, &option_index );
+
+        if( option < 0 ) break;
+
+        switch( option )
+        {
+            case 0 :
+
+                break;
+
+            case 'a' :
+
+                G.channels = a_chans;
+                break;
+
+            case 'g' :              
+            case 'b' :
+
+                G.channels = bg_chans;                      
+                break;
+
+            case 'c' :
+
+                sscanf(optarg, "%d",&(G.channel));
+                
+                if ( G.channel < 0 )
+                    goto usage;
+
+                if (G.channel != 0)
+                {
+                    n=0;
+                    do
+                    {
+                        if (G.channel == abg_chans[n])
+                            break;
+                    }
+                    while (abg_chans[++n]);
+                    if (G.channel != abg_chans[n])
+                        goto usage;
+                }
+                /*
+                 * else, hop between channels
+                 * so, no need to break
+				 */
+            case 'h' :
+                
+                /* Default behaviour, hop on all available channels */
+                G.channels     =  abg_chans;
+                break;
+
+            case 'i':
+
+                ivs_only = 1;
+                break;
+
+            case 'p':
+            
+                G.usegpsd  = 1;
+                /*
+                if (inet_aton(optarg, &provis_addr.sin_addr) == 0 )
+                {
+                    printf("Invalid IP address.\n");
+                    return (1);
+                }
+                */
+                break;
+
+            case 'w':
+            
+                /* Write prefix */
+                dump_prefix = optarg;
+                break;
+
+            default : goto usage;
+        }
+    }
+    while ( 1 );
+
+    if ( dump_prefix == NULL)
+    {
+        printf("Missing dump prefix.\n");
+        return (1);
+    }
+
+    /* Check if we have root privileges */
+
+    if( geteuid() != 0 )
+    {
+        printf( "This program requires root privileges.\n" );
+        return( 1 );
+    }
 
     /* create the raw socket and drop privileges */
 
@@ -1614,12 +1938,21 @@ int main( int argc, char *argv[] )
     if( fd_raw < 0 )
     {
         perror( "socket(PF_PACKET) failed" );
-        fprintf( stderr, "This program requires root privileges.\n" );
+        if( getuid() != 0 )
+            fprintf( stderr, "This program requires root privileges.\n" );
         return( 1 );
     }
-
+    
     setuid( getuid() );
 
+    /* Check iwpriv existence */
+
+    if (! iwpriv_exist() )
+    {
+        printf("Can't find wireless tools, exiting.\n");
+        return (1);
+    }
+
     /* reserve the buffer space */
 
     if( ( buffer = (unsigned char *) malloc( 65536 ) ) == NULL )
@@ -1630,26 +1963,36 @@ int main( int argc, char *argv[] )
 
     /* find the interface index */
 
+    iface = argv[argc-1];
+
     memset( &ifr, 0, sizeof( ifr ) );
-    strncpy( ifr.ifr_name, argv[1], sizeof( ifr.ifr_name ) - 1 );
+    strncpy( ifr.ifr_name, iface, sizeof( ifr.ifr_name ) - 1 );
 
     if( ioctl( fd_raw, SIOCGIFINDEX, &ifr ) < 0 )
     {
-        fprintf( stderr, "%s is not a network interface.\n", argv[1] );
+        fprintf( stderr, "%s is not a network interface.\n", argv[argc-1] );
         return( 1 );
     }
 
+    /* Exit if ndiswrapper : check iwpriv ndis_reset */
+    
+    if ( is_ndiswrapper(iface) )
+    {
+        printf("Ndiswrapper doesn't support monitor mode.\n");
+        return (1);
+    }
+
     memset( &sll, 0, sizeof( sll ) );
     sll.sll_family   = AF_PACKET;
     sll.sll_ifindex  = ifr.ifr_ifindex;
     sll.sll_protocol = htons( ETH_P_ALL );
 
-    if( memcmp( argv[1], "wlan", 4 ) == 0 )
+    if( memcmp( iface, "wlan", 4 ) == 0 )
     {
         if( ( pid = fork() ) == 0 )     /* wlan-ng brain damage */
         {
             close( 0 ); close( 1 ); close( 2 ); chdir( "/" );
-            execlp( "wlanctl-ng", "wlanctl-ng", argv[1],
+            execlp( "wlanctl-ng", "wlanctl-ng", iface,
                     "lnxreq_ifstate", "ifstate=enable", NULL );
             exit( 1 );
         }
@@ -1662,7 +2005,7 @@ int main( int argc, char *argv[] )
         if( ! fork() )                  /* hostap card reset */
         {
             close( 0 ); close( 1 ); close( 2 ); chdir( "/" );
-            execlp( "iwpriv", "iwpriv", argv[1], "reset", "1", NULL );
+            execlp( "iwpriv", "iwpriv", iface, "reset", "1", NULL );
             exit( 1 );
         }
         wait( NULL );
@@ -1670,12 +2013,12 @@ int main( int argc, char *argv[] )
 
     /* test if orinoco */
 
-    if( memcmp( argv[1], "eth", 3 ) == 0 )
+    if( memcmp( iface, "eth", 3 ) == 0 )
     {
         if( ( pid = fork() ) == 0 )
         {
             close( 0 ); close( 1 ); close( 2 ); chdir( "/" );
-            execlp( "iwpriv", "iwpriv", argv[1], "get_port3", NULL );
+            execlp( "iwpriv", "iwpriv", iface, "get_port3", NULL );
             exit( 1 );
         }
 
@@ -1685,6 +2028,30 @@ int main( int argc, char *argv[] )
             G.is_orinoco = 1;
     }
 
+    /* Check if madwifi-ng */
+
+    G.is_madwifing = 0;
+    memset( buffer,0, 65536 );
+    snprintf( (char*) buffer, strlen( iface ) + 23,
+        "/proc/sys/net/%s/%%parent", iface );
+    check_madwifing = fopen( (char*) buffer,"r");
+    if (check_madwifing != NULL)
+    {
+        fclose(check_madwifing);
+        G.is_madwifing = 1;
+        memset(buffer,0, 65536);
+        sprintf((char *) buffer, "/proc/sys/net/%s/dev_type", iface);
+        FILE * f = fopen(buffer,"r");
+        if (f != NULL)
+        {
+            char * madwifiPrism2 = "802";
+            fwrite(madwifiPrism2, 3, 1, f);
+            fclose(f);
+        }
+        /* Force prism2 header on madwifi-ng */
+    }
+    memset(buffer,0, 65536);
+
     /* make sure the interface is up */
 
     ifr.ifr_flags = IFF_UP | IFF_BROADCAST | IFF_RUNNING;        
@@ -1695,7 +2062,11 @@ int main( int argc, char *argv[] )
         return( 1 );
     }
 
-    set_monitor( argv[1], fd_raw );
+    if (set_monitor( iface, fd_raw ))
+    {
+        printf("Error setting monitor mode on %s\n",iface);
+        return( 1 );
+    }
 
     /* bind the raw socket to the interface */
 
@@ -1711,7 +2082,7 @@ int main( int argc, char *argv[] )
     if( ! fork() )  /* hostap */
     {
         close( 0 ); close( 1 ); close( 2 ); chdir( "/" );
-        execlp( "iwpriv", "iwpriv", argv[1], "monitor_type", "1", NULL );
+        execlp( "iwpriv", "iwpriv", iface, "monitor_type", "1", NULL );
         exit( 1 );
     }
     wait( NULL );
@@ -1719,7 +2090,7 @@ int main( int argc, char *argv[] )
     if( ! fork() )  /* r8180 */
     {
         close( 0 ); close( 1 ); close( 2 ); chdir( "/" );
-        execlp( "iwpriv", "iwpriv", argv[1], "prismhdr", "1", NULL );
+        execlp( "iwpriv", "iwpriv", iface, "prismhdr", "1", NULL );
         exit( 1 );
     }
     wait( NULL );
@@ -1727,14 +2098,14 @@ int main( int argc, char *argv[] )
     if( ! fork() )  /* prism54 */
     {
         close( 0 ); close( 1 ); close( 2 ); chdir( "/" );
-        execlp( "iwpriv", "iwpriv", argv[1], "set_prismhdr", "1", NULL );
+        execlp( "iwpriv", "iwpriv", iface, "set_prismhdr", "1", NULL );
         exit( 1 );
     }
     wait( NULL );
 
     /* make sure the WEP key is off */
 
-    disable_wep_key( argv[1], fd_raw );
+    disable_wep_key( iface, fd_raw );
 
     /* start a child to hop between channels */
 
@@ -1746,26 +2117,12 @@ int main( int argc, char *argv[] )
 
         if( ! fork() )
         {
-            channel_hopper( argv[1], fd_raw );
+            channel_hopper( iface, fd_raw );
             exit( 1 );
         }
     }
     else
-        set_channel( argv[1], fd_raw, G.channel );
-
-    /* start the GPS tracker */
-
-    pipe( G.gc_pipe );
-    signal( SIGUSR2, sighandler );
-
-    if( ! fork() )
-    {
-        gps_tracker();
-        exit( 1 );
-    }
-
-    usleep( 50000 );
-    waitpid( -1, NULL, WNOHANG );
+        set_channel( iface, fd_raw, G.channel );
 
     /* lookup the hardware type */
 
@@ -1790,7 +2147,7 @@ int main( int argc, char *argv[] )
         fprintf( stderr, "- expected ARPHRD_IEEE80211\nor ARPHRD_IEEE8021"
                          "1_PRISM instead.  Make sure RFMON is enabled:\n"
                          "run 'ifconfig %s up; iwconfig %s mode Monitor "
-                         "channel <#>'\n\n", argv[1], argv[1] );
+                         "channel <#>'\n\n", iface, iface );
         return( 1 );
     }
 
@@ -1809,7 +2166,7 @@ int main( int argc, char *argv[] )
 
     /* open or create the output files */
 
-    if( dump_initialize( argv[2], ivs_only ) )
+    if( dump_initialize( dump_prefix, ivs_only ) )
         return( 1 );
 
     signal( SIGINT,   sighandler );
@@ -1819,13 +2176,34 @@ int main( int argc, char *argv[] )
 
     sighandler( SIGWINCH );
 
+    /* start the GPS tracker */
+
+    if (G.usegpsd)
+    {
+        pipe( G.gc_pipe );
+        signal( SIGUSR2, sighandler );
+
+        if( ! fork() )
+        {
+            gps_tracker();
+            exit( 1 );
+        }
+
+        usleep( 50000 );
+        waitpid( -1, NULL, WNOHANG );
+    }
+
     fprintf( stderr, "\33[?25l\33[2J\n" );
 
-    update_battery();
+	start_time = time( NULL );
+    tt1        = time( NULL );
+    tt2        = time( NULL );
+    tt3        = time( NULL );
 
-    tt1 = time( NULL );
-    tt2 = time( NULL );
-    tt3 = time( NULL );
+	G.batt     = getBatteryString();
+
+	G.elapsed_time = (char *) calloc( 1, 4 );
+	strcpy(G.elapsed_time,"0 s");
 
     while( 1 )
     {
@@ -1846,9 +2224,17 @@ int main( int argc, char *argv[] )
         if( time( NULL ) - tt2 > 3 )
         {
             /* update the battery state */
+			free(G.batt);
+			
+            tt2 			= time( NULL );
+            G.batt 			= getBatteryString();
+            
+            /* update elapsed time */
+
+			free(G.elapsed_time);
+    		G.elapsed_time = getStringTimeFromSec( 
+    			difftime(tt2, start_time) );
 
-            tt2 = time( NULL );
-            update_battery();
 
             /* flush the output files */
 
@@ -1917,7 +2303,7 @@ int main( int argc, char *argv[] )
 
         /* if device is an atheros, remove the FCS */
 
-        if( ! memcmp( argv[1], "ath", 3 ) )
+        if( ! memcmp( iface, "ath", 3 ) && (! G.is_madwifing) )
             caplen -= 4;
 
         /* prism (wlan-ng) header parsing */
@@ -1938,7 +2324,7 @@ int main( int argc, char *argv[] )
             {
                 power = *(int *)( buffer + 0x5C );
 
-                if( ! memcmp( argv[1], "ath", 3 ) )
+                if( ! memcmp( iface, "ath", 3 ) )
                     power -= *(int *)( buffer + 0x68 );
 
                 n = *(int *)( buffer + 4 );
